@@ -17,8 +17,8 @@ fn main() {
   build_device_boot(&out_dir);
 }
 
-#[cfg(feature="atmega4809")]
-fn build_device_boot(out_dir: &String) {
+#[cfg(all(feature="atmega4809",feature="bootable"))]
+fn build_device_boot(out_dir: &str) {
 
   let _result = Command::new("avr-as")
     .args(&["src/hal/atmega4809/boot.S",
