@@ -22,6 +22,18 @@ use core::cell::UnsafeCell;
 extern crate alloc;
 #[cfg(target_arch="avr")]
 pub use alloc::vec;
+#[cfg(target_arch="avr")]
+pub use alloc::boxed;
+#[cfg(target_arch="avr")]
+pub use alloc::rc;
+
+#[cfg(not(target_arch="avr"))]
+pub use std::boxed;
+#[cfg(not(target_arch="avr"))]
+pub use std::vec;
+#[cfg(not(target_arch="avr"))]
+pub use std::rc;
+
 
 // Declarations ==============================================================
 
@@ -77,13 +89,19 @@ pub fn initialise() {
  * Implementation of the Rust GlobalAlloc trait for our First-Fit allocator,
  * allowing it to be used as the Rust global allocator.
  */
+#[optimize(speed)]
 unsafe impl<const H:usize, const S:usize, const S2:usize> GlobalAlloc for GlobalAllocator<H,S,S2> {
   unsafe fn alloc(&self, layout: Layout) -> *mut u8 {
     let heap = crate::panic_if_none!(self.heap.get().as_mut());
 
     // Calculate the size we need to allocate for the 'worst case' amount of
     // padding needed to achieve the requested alignment
-    let align_words = (layout.align()/2)-1;
+    let align_words = match layout.align() {
+      0 => 0,
+      1 => 0,
+      _other => (layout.align()/2)-1
+    };
+
     let size_words=  ((layout.size() + 1) / 2) + align_words;
 
     // Allocate, including space for padding if needed
@@ -420,6 +438,7 @@ impl<const HEAPSIZE_WORDS: usize, const SEGS: usize, const S21: usize> FirstFitH
 // Tests =====================================================================
 #[cfg(test)]
 mod test {
+  use std::vec::Vec;
   use crate::alloc::*;
 
   #[test]
@@ -512,18 +531,18 @@ mod test {
     assert_eq!(heap.bl_available(), init_free);
   }
 
-  fn fillspace(addr: *mut u8, size: usize) {
+  fn fillspace(addr: *mut u8, size: usize, value: u8) {
     unsafe {
       for i in 0..size as isize {
-        *(addr.offset(i)) = 0xde;
+        *(addr.offset(i)) = value;
       }
     }
   }
 
-  fn checkspace(addr: *mut u8, size: usize) -> bool {
+  fn checkspace(addr: *mut u8, size: usize, value: u8) -> bool {
     unsafe {
       for i in 0..size as isize {
-        if *(addr.offset(i)) != 0xde {
+        if *(addr.offset(i)) != value {
           return false;
         }
       }
@@ -545,7 +564,7 @@ mod test {
 
       // Do an allocation
       let addr = global.alloc(Layout::from_size_align(16,2).unwrap());
-      fillspace(addr, 16);
+      fillspace(addr, 16,0xde);
       println!("Allocated a block @ address: {:?}", addr);
       println!("Free space: {}b", heap.bl_available()*2);
       assert_eq!(addr as usize % 2, 0); // Check the returned pointer has correct alignment
@@ -559,35 +578,76 @@ mod test {
       println!("Alignment checks");
       let discard = global.alloc(Layout::from_size_align(3, 2).unwrap());
       let addr1 = global.alloc(Layout::from_size_align(175,4).unwrap());
-      fillspace(addr1, 175);
+      fillspace(addr1, 175, 0xde);
       println!("Allocated a block @ address: {:?}", addr1);
       println!("Free space: {}b", heap.bl_available()*2);
       assert_eq!(addr1 as usize % 4, 0); // Check the returned pointer has correct alignment
 
       let addr2 = global.alloc(Layout::from_size_align(230, 8).unwrap());
-      fillspace(addr2, 230);
+      fillspace(addr2, 230, 0xde);
       println!("Allocated a block @ address: {:?}", addr2);
       println!("Free space: {}b", heap.bl_available()*2);
       assert_eq!(addr2 as usize % 8, 0); // Check the returned pointer has correct alignment
 
       let addr3 = global.alloc(Layout::from_size_align(49, 16).unwrap());
-      fillspace(addr3, 49);
+      fillspace(addr3, 49, 0xde);
       println!("Allocated a block @ address: {:?}", addr3);
       println!("Free space: {}b", heap.bl_available()*2);
       assert_eq!(addr3 as usize % 16, 0); // Check the returned pointer has correct alignment
 
       println!("Checking for data corruption");
-      assert!(checkspace(addr1, 175));
-      assert!(checkspace(addr2, 230));
-      assert!(checkspace(addr3, 49));
+      assert!(checkspace(addr1, 175,0xde));
+      assert!(checkspace(addr2, 230,0xde));
+      assert!(checkspace(addr3, 49,0xde));
 
       println!("Freeing aligned allocations");
       global.dealloc(addr1, Layout::from_size_align(175,4).unwrap());
-      assert!(checkspace(addr2, 230));
-      assert!(checkspace(addr3, 49));
+      assert!(checkspace(addr2, 230,0xde));
+      assert!(checkspace(addr3, 49,0xde));
       global.dealloc(addr2, Layout::from_size_align(230, 8).unwrap());
-      assert!(checkspace(addr3, 49));
+      assert!(checkspace(addr3, 49,0xde));
       global.dealloc(addr3, Layout::from_size_align(49, 16).unwrap());
     }
   }
+
+  #[test]
+  pub fn test_soaktest_global_alloc() {
+    unsafe {
+      let global = GlobalAllocator {
+        heap: UnsafeCell::new(FirstFitHeap::<512,32,64>::new())
+      };
+
+      let mut heap = global.heap.get().as_mut().unwrap();
+
+      heap.init();
+
+      for blocksize in 1..1000 {
+        println!("Testing blocks of size {}", blocksize);
+
+        let mut blocks : Vec<*mut u8> = Vec::new();
+
+        let mut i:i32 = 0;
+        loop {
+          let block = global.alloc(Layout::from_size_align(blocksize,1).unwrap());
+
+          if block != core::ptr::null_mut() {
+            fillspace(block,blocksize,i as u8);
+            blocks.push(block);
+          } else {
+            break;
+          }
+          i += 1;
+        }
+
+        println!("  Allocated {} blocks", blocks.len());
+        let mut i:i32 = 0;
+        for block in blocks {
+          assert!(checkspace(block,blocksize,i as u8));
+          global.dealloc(block,Layout::from_size_align(blocksize,1).unwrap());
+          i += 1;
+        }
+      }
+    }
+  }
+
 }
\ No newline at end of file
