@@ -7,13 +7,14 @@
 
 // Imports ===================================================================
 use crate::hal::generic::port::{Pin, PinMode, InterruptMode};
+use crate::util::OwnOrBorrowMut;
 
 // Declarations ==============================================================
 pub struct Led<P>
 where
   P: 'static + Pin
 {
-  pin: &'static mut P
+  pin: OwnOrBorrowMut<'static,P>
 }
 
 // Code ======================================================================
@@ -21,28 +22,34 @@ impl<P> Led<P>
 where
   P: Pin
 {
-  pub fn using_pin(pin: &'static mut P) -> Self {
+  pub fn using<OP: Into<OwnOrBorrowMut<'static, P>>>(pin: OP) -> Self {
+    let pin : OwnOrBorrowMut<P> = pin.into();
+
     pin.set_mode(PinMode::Output);
     pin.set_interrupt_mode(InterruptMode::Disabled);
     Led { pin }
   }
 
-  pub fn set_on(&mut self) {
+  pub fn with_pin(pin: &'static mut P) -> Self {
+    Self::using(pin)
+  }
+
+  pub fn set_on(&self) {
     self.pin.set_high();
   }
 
-  pub fn set_off(&mut self) {
+  pub fn set_off(&self) {
     self.pin.set_low();
   }
 
-  pub fn set(&mut self, on: bool) {
+  pub fn set(&self, on: bool) {
     match on {
       true => self.set_on(),
       false => self.set_off()
     }
   }
 
-  pub fn toggle(&mut self) {
+  pub fn toggle(&self) {
     self.pin.toggle()
   }
 }
