@@ -26,6 +26,7 @@ use crate::stdout;
 #[panic_handler]
 #[cfg(target_arch="avr")]
 fn panic(info: &core::panic::PanicInfo) -> ! {
+  #![allow(deprecated)] // until asm! is implemented for AVR
   unsafe {
     crate::hal::concurrency::interrupt::disable_interrupts();
 
@@ -61,6 +62,9 @@ fn panic(info: &core::panic::PanicInfo) -> ! {
       }
     }
 
+
+    llvm_asm!("break" :::: "volatile");
+
     loop {
       crate::hal::atmega4809::debugled::PinE2DebugLed::toggle();
       for _i in 0..255 {
@@ -87,4 +91,5 @@ const ADDR_TCB0:    u16 = 0x0A80;
 const ADDR_TCB1:    u16 = 0x0A90;
 const ADDR_TCB2:    u16 = 0x0AA0;
 const ADDR_TCB3:    u16 = 0x0AB0;
-const ADDR_WDT:     u16 = 0x0100;
\ No newline at end of file
+const ADDR_WDT:     u16 = 0x0100;
+const ADDR_RTC:     u16 = 0x0140;
\ No newline at end of file
