@@ -5,7 +5,7 @@
  */
 //! Generic traits for the various devices that can be attached to our AVR
 //! device
-
+pub mod callback;
 pub mod timer;
 pub mod port;
 pub mod debugled;
@@ -18,6 +18,7 @@ pub mod serial;
 #[cfg(target_arch="avr")]
 pub(in crate) fn busy_loop(mut c: u16) {
   unsafe {
+    #![allow(deprecated)] // until asm! is implemented for AVR
     llvm_asm!("1: sbiw $0,1\n\tbrne 1b"
                      : "=w"(c)
                      : "0"(c)
