@@ -9,6 +9,9 @@
 
 
 
+use core::any::Any;
+use crate::hal::generic::callback::IsrCallback;
+
 // Declarations ==============================================================
 /**
  * Input/output mode for a pin
@@ -70,34 +73,35 @@ pub enum PinIdentity {
  * using mutual exclusion where necessary, and *DO NOT DO HEAVY PROCESSING
  * IN THE CALLBACK*.
  */
-pub type PinInterruptHandler = fn(PinIdentity,bool) -> ();
+pub type PinIsrFunction<P> = fn(&'static P,PinIdentity,bool,Option<*const dyn Any>) -> ();
+
+pub type PinIsrCallback<P> = IsrCallback<PinIsrFunction<P>,()>;
 
 pub trait Pin {
   /// Set the input/output mode for this pin
-  fn set_mode(&mut self, mode: PinMode);
+  fn set_mode(&self, mode: PinMode);
 
   /// Toggle this pin's output level
-  fn toggle(&mut self);
+  fn toggle(&self);
 
   /// Set the pin's output to high
-  fn set_high(&mut self);
+  fn set_high(&self);
 
   /// Set the pin's output to low
-  fn set_low(&mut self);
+  fn set_low(&self);
 
   /// Set the pin's output according to the boolean (true == high, false == low)
-  fn set(&mut self, high: bool);
+  fn set(&self, high: bool);
 
   /// Get the pin's input; true == high, false == low.
   fn get(&self) -> bool;
 
   /// Set when this pin will generate interrupts
-  fn set_interrupt_mode(&mut self, mode: InterruptMode);
+  fn set_interrupt_mode(&self, mode: InterruptMode);
 
   /// Listen for interrupts on this pin and call the given handler callback
   /// when one occurs.
-  fn listen(&mut self, handler: Option<PinInterruptHandler>);
-
+  fn listen(&'static self, handler: PinIsrCallback<Self>);
 }
 
 pub trait Port<P>
@@ -269,23 +273,15 @@ pub mod base {
   #[macro_export]
   macro_rules! atmel_port_tpl {
     ($ref:expr, $portsrc:expr, $isr:ident) => {
+      use core::any::Any;
 
-
-      use crate::hal::generic::port::{PinMode,Pin,PinIdentity,InterruptMode,PinInterruptHandler};
+      use crate::hal::generic::port::{PinMode,Pin,PinIdentity,InterruptMode,PinIsrCallback};
       use crate::hal::generic::port::base::{AtmelPortControl,AvrPortRegisterBlock};
+      use crate::isr_cb_invoke;
 
       pub type PortImpl = AvrPortRegisterBlock;
       pub type PinImpl = AtmelPin;
 
-      /**
-       * A no-op function that is the default handler for the button
-       * interrupt callbacks.
-       */
-      #[inline(never)]
-      fn nop_handler(_src: PinIdentity, _state: bool) -> () {
-        unsafe { llvm_asm!("nop" :::: "volatile" ) }
-      }
-
       /**
        * A single pin instance.  This is just a wrapper for a reference to
        * the pin number and a place to stash the callback function that will
@@ -293,7 +289,7 @@ pub mod base {
        */
       pub struct AtmelPin {
         n: u8,
-        handler: PinInterruptHandler
+        handler: PinIsrCallback<Self>
       }
 
       /**
@@ -301,7 +297,7 @@ pub mod base {
        */
       impl Pin for AtmelPin {
         #[inline(always)]
-        fn set_mode(&mut self, mode: PinMode) {
+        fn set_mode(&self, mode: PinMode) {
           match mode {
             PinMode::Output => {
               instance().enable_output(self.n);
@@ -317,19 +313,19 @@ pub mod base {
           }
         }
         #[inline(always)]
-        fn toggle(&mut self) {
+        fn toggle(&self) {
           instance().toggle(self.n);
         }
         #[inline(always)]
-        fn set_high(&mut self) {
+        fn set_high(&self) {
           instance().set_high(self.n);
         }
         #[inline(always)]
-        fn set_low(&mut self) {
+        fn set_low(&self) {
           instance().set_low(self.n);
         }
         #[inline(always)]
-        fn set(&mut self, high: bool) {
+        fn set(&self, high: bool) {
           match high {
             true  => instance().set_high(self.n),
             false => instance().set_low(self.n)
@@ -339,16 +335,13 @@ pub mod base {
         fn get(&self) -> bool {
           instance().get(self.n)
         }
-        fn set_interrupt_mode(&mut self, mode: InterruptMode) {
+        fn set_interrupt_mode(&self, mode: InterruptMode) {
           instance().set_interrupt_mode(self.n, mode)
         }
-        fn listen(&mut self, handler: Option<PinInterruptHandler>){
+        fn listen(&self, handler: PinIsrCallback<Self>){
           crate::hal::concurrency::interrupt::isolated(||{
             unsafe {
-              PINS[self.n as usize].handler = match handler {
-                Some(handler) => handler,
-                None => nop_handler
-              }
+              PINS[self.n as usize].handler = handler;
             }
           });
         }
@@ -373,7 +366,7 @@ pub mod base {
           if(!INITIALISED){
             for pin in 0..=7 {
               PINS[pin].n = pin as u8;
-              PINS[pin].handler = nop_handler;
+              PINS[pin].handler = PinIsrCallback::Nop(());
             }
             INITIALISED = true;
           }
@@ -383,14 +376,14 @@ pub mod base {
       }
 
       static mut PINS : [AtmelPin; 8] = [
-        AtmelPin { n: 0, handler: nop_handler },
-        AtmelPin { n: 1, handler: nop_handler },
-        AtmelPin { n: 2, handler: nop_handler },
-        AtmelPin { n: 3, handler: nop_handler },
-        AtmelPin { n: 4, handler: nop_handler },
-        AtmelPin { n: 5, handler: nop_handler },
-        AtmelPin { n: 6, handler: nop_handler },
-        AtmelPin { n: 7, handler: nop_handler },
+        AtmelPin { n: 0, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 1, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 2, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 3, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 4, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 5, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 6, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 7, handler: PinIsrCallback::Nop(()) },
       ];
 
 
@@ -398,11 +391,7 @@ pub mod base {
        * Return a single pin instance wrapper.
        */
       pub fn pin_instance(pin: u8) -> &'static mut AtmelPin {
-        //debug_assert!(pin < 8);
-
         unsafe {
-          PINS[0].handler = nop_handler;
-
           &mut PINS[pin as usize]
         }
       }
@@ -420,7 +409,7 @@ pub mod base {
 
           for pin in 0..=7 {
             if (intflags & mask) > 0 {
-              (PINS[pin].handler)($portsrc(pin as u8), (state & mask) > 0);
+              isr_cb_invoke!(PINS[pin].handler, &mut PINS[pin], $portsrc(pin as u8), (state & mask) > 0);
             }
             mask <<= 1;
           }
@@ -539,9 +528,9 @@ pub mod base {
   #[macro_export]
   macro_rules! atmel_port_tpl {
     ($ref:expr, $portsrc:expr, $isr:ident) => {
+      use core::any::Any;
 
-
-      use crate::hal::generic::port::{PinMode,Pin,PinIdentity,InterruptMode,PinInterruptHandler};
+      use crate::hal::generic::port::{PinMode,Pin,PinIdentity,InterruptMode,PinIsrCallback};
       use crate::hal::generic::port::base::{AtmelPortControl,DummyPortRegisterBlock};
 
       pub type PortImpl = DummyPortRegisterBlock;
@@ -554,14 +543,7 @@ pub mod base {
        */
       pub struct AtmelPin {
         n: u8,
-        handler: PinInterruptHandler
-      }
-
-      /**
-       * A no-op function that is the default handler for the button
-            * interrupt callbacks.
-       */
-      fn nop_handler(_src: PinIdentity, _state: bool) -> () {
+        handler: PinIsrCallback<Self>
       }
 
       /**
@@ -569,7 +551,7 @@ pub mod base {
        */
       impl Pin for AtmelPin {
         #[inline(always)]
-        fn set_mode(&mut self, mode: PinMode) {
+        fn set_mode(&self, mode: PinMode) {
           match mode {
             PinMode::Output => {
               instance().enable_output(self.n);
@@ -585,19 +567,19 @@ pub mod base {
           }
         }
         #[inline(always)]
-        fn toggle(&mut self) {
+        fn toggle(&self) {
           instance().toggle(self.n);
         }
         #[inline(always)]
-        fn set_high(&mut self) {
+        fn set_high(&self) {
           instance().set_high(self.n);
         }
         #[inline(always)]
-        fn set_low(&mut self) {
+        fn set_low(&self) {
           instance().set_low(self.n);
         }
         #[inline(always)]
-        fn set(&mut self, high: bool) {
+        fn set(&self, high: bool) {
           match high {
             true  => instance().set_high(self.n),
             false => instance().set_low(self.n)
@@ -607,16 +589,13 @@ pub mod base {
         fn get(&self) -> bool {
           instance().get(self.n)
         }
-        fn set_interrupt_mode(&mut self, mode: InterruptMode) {
+        fn set_interrupt_mode(&self, mode: InterruptMode) {
           instance().set_interrupt_mode(self.n, mode)
         }
-        fn listen(&mut self, handler: Option<PinInterruptHandler>){
+        fn listen(&self, handler: PinIsrCallback<Self>){
           crate::hal::concurrency::interrupt::isolated(||{
             unsafe {
-              PINS[self.n as usize].handler = match handler {
-                Some(handler) => handler,
-                None => nop_handler
-              }
+              PINS[self.n as usize].handler = handler;
             }
           });
         }
@@ -647,7 +626,7 @@ pub mod base {
           if(!INITIALISED){
             for pin in 0..=7 {
               PINS[pin].n = pin as u8;
-              PINS[pin].handler = nop_handler;
+              PINS[pin].handler = PinIsrCallback::Nop(());
             }
             INITIALISED = true;
           }
@@ -657,14 +636,14 @@ pub mod base {
       }
 
       static mut PINS : [AtmelPin; 8] = [
-        AtmelPin { n: 0, handler: nop_handler },
-        AtmelPin { n: 1, handler: nop_handler },
-        AtmelPin { n: 2, handler: nop_handler },
-        AtmelPin { n: 3, handler: nop_handler },
-        AtmelPin { n: 4, handler: nop_handler },
-        AtmelPin { n: 5, handler: nop_handler },
-        AtmelPin { n: 6, handler: nop_handler },
-        AtmelPin { n: 7, handler: nop_handler },
+        AtmelPin { n: 0, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 1, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 2, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 3, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 4, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 5, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 6, handler: PinIsrCallback::Nop(()) },
+        AtmelPin { n: 7, handler: PinIsrCallback::Nop(()) },
       ];
 
 
@@ -672,11 +651,7 @@ pub mod base {
        * Return a single pin instance wrapper.
        */
       pub fn pin_instance(pin: u8) -> &'static mut AtmelPin {
-        //debug_assert!(pin < 8);
-
         unsafe {
-          PINS[0].handler = nop_handler;
-
           &mut PINS[pin as usize]
         }
       }
