@@ -26,18 +26,4 @@ pub trait Write {
 // Code ======================================================================
 
 
-// Tests =====================================================================
-#[cfg(test)]
-mod tests {
-  #[allow(unused_imports)]
-  use super::*;
-
-  #[test]
-  fn a_test() {}
-
-  #[test]
-  #[should_panic]
-  fn a_failure_test() {
-    panic!()
-  }
-}
\ No newline at end of file
+// Tests =====================================================================
\ No newline at end of file
