@@ -94,30 +94,52 @@
 //! |               | `alloc_large`  | 3072 bytes |
 //!
 //! # A Minimal AVRoxide Program
+//! This program shows how to access the devices using the Arduino aliases
+//! for an Arduino Nano Every, and how to listen to button events from a
+//! button attached to pin A2 to toggle an LED attached to pin D7 every time
+//! the button is pressed.
+//!
+//! We also show the use of a software debouncer on the pin, and configuring
+//! a serial port.
+//!
 //! ```no_run
 //! #![no_std]
 //! #![no_main]
 //!
 //! use avr_oxide::hal::atmega4809::hardware;
+//! use avr_oxide::alloc::boxed::Box;
+//! use avr_oxide::devices::debouncer::Debouncer;
+//! use avr_oxide::devices::{ OxideLed, OxideButton, OxideSerialPort };
+//! use avr_oxide::hal::generic::serial::{BaudRate, DataBits, Parity, SerialPortMode, StopBits};
+//! use avr_oxide::io::Write;
+//! use avr_oxide::arduino;
 //!
 //! #[avr_oxide::main(chip="atmega4809")]
 //! pub fn main() {
-//!   let hardware = hardware::instance();
 //!   let supervisor = avr_oxide::oxide::instance();
+//!   let hardware = hardware::instance();
+//!   let arduino = arduino::nanoevery::Arduino::from(hardware);
 //!
-//!   supervisor.run(|event| {
-//!     match event {
-//!       avr_oxide::event::OxideEvent::Initialise => {
-//!         // Do some inititialisation
-//!       },
-//!       _ => {
-//!        // Ignore anything else
-//!       }
-//!     }
-//!   });
+//!   // Configure the serial port early so we can get any panic!() messages
+//!   let mut serial= OxideSerialPort::using_port_and_pins(arduino.usb_serial,
+//!                                                        arduino.usb_serial_tx,
+//!                                                        arduino.usb_serial_rx).mode(SerialPortMode::Asynch(BaudRate::Baud9600, DataBits::Bits8, Parity::None, StopBits::Bits1));
+//!   serial.write(b"Welcome to AVRoxide\n");
+//!
+//!   let green_led = OxideLed::with_pin(arduino.d7);
+//!   let mut green_button = Box::new(OxideButton::using(Debouncer::with_pin(arduino.a2)));
+//!
+//!   green_button.on_click(Box::new(move |_pinid, _state|{
+//!     green_led.toggle();
+//!   }));
+//!
+//!   // Tell the supervisor which devices to listen to
+//!   supervisor.listen(green_button);
+//!
+//!   // Now enter the event loop
+//!   supervisor.run();
 //! }
 //! ```
-#![feature(int_bits_const)]
 #![feature(llvm_asm)]
 #![feature(abi_avr_interrupt)]
 #![feature(naked_functions)]
@@ -129,6 +151,9 @@
 #![feature(maybe_uninit_uninit_array)]
 #![feature(maybe_uninit_extra)]
 #![feature(alloc_error_handler)]
+#![feature(dropck_eyepatch)]
+#![feature(trait_alias)]
+#![feature(optimize_attribute)]
 
 // no_std if built for AVR, if built for anything else it'll be for testing
 // and then we do want std
@@ -141,6 +166,7 @@ pub mod hal;
 pub mod event;
 pub mod oxide;
 pub mod deviceconsts;
+pub mod util;
 pub(crate) mod private;
 mod mutstatic;
 
