@@ -21,14 +21,17 @@ macro_rules! mut_singleton {
     #[inline(always)]
     pub fn $accessor() -> &'static mut $type {
       unsafe {
-        if $name.is_none() {
-          core::ptr::replace(&mut $name, Some($constructor));
-        }
 
-        match &mut $name {
-          None => panic!(),
-          Some(instance) => instance
-        }
+        crate::hal::concurrency::interrupt::isolated(||{
+          if $name.is_none() {
+            core::ptr::replace(&mut $name, Some($constructor));
+          }
+
+          match &mut $name {
+            None => panic!(),
+            Some(instance) => instance
+          }
+        })
       }
     }
   }
